@@ -4,14 +4,17 @@ use warnings;
 use Data::Dumper;
 my (
     @meta,
+    %range,                     # base types.
     %name_to_value,             # just the names in the srl_protocol.h
     %name_to_value_expanded,    # names from srl_protocol, but with the LOW/HIGH data expanded
     %value_to_name_expanded,    # values from srl_protocol_expanded, mapping back, note value points at FIRST name
     %value_to_comment_expanded  # values from srl_protocol_expanded, with comments from file.
 );
 my $max_name_length= 0;
+my %define;
+my %define_is_str;
 
-sub fill_ranges {
+sub fill_range {
     my $pfx= shift;
     $pfx=~s/_LOW//;
     defined(my $ofs= $name_to_value_expanded{$pfx})
@@ -27,12 +30,15 @@ sub fill_ranges {
         $meta[$value]{value}= $value;
         $meta[$value]{type_name}= $pfx;
         $meta[$value]{type_value}= $ofs;
+
+        push @{$range{$pfx}}, $meta[$value];
         #$meta[$value]{comment}= $value_to_comment_expanded{ $ofs }
         #    if exists $value_to_comment_expanded{ $ofs };
 
         $meta[$value]{masked_val}= $n;
         $meta[$value]{masked}= 1;
 
+        #$define{"SRL_HDR_".$name}= $value;
     }
     $value_to_comment_expanded{ $name_to_value_expanded{$pfx . "_HIGH"} } = $value_to_comment_expanded{ $ofs };
 }
@@ -42,8 +48,10 @@ sub read_protocol {
 
     my @fill;
     while (<$fh>) {
-        if(m!^#define\s+SRL_HDR_(\S+)\s+\(\(char\)(\d+)\)\s*(?:/\*\s*(.*?)\s*\*/)?\s*\z!i) {
-            my ($name, $value, $comment)= ($1, $2, $3);
+        chomp;
+        my $orig= $_;
+        if(m!^#define\s+(SRL_HDR_(\S+))\s+\(\(U8\)(\d+)\)\s*(?:/\*\s*(.*?)\s*\*/)?\s*\z!i) {
+            my ($full_name, $name, $value, $comment)= ($1, $2, $3, $4);
             $value= 0+$value;
             $name_to_value{$name}= $value;
             $name_to_value_expanded{$name}= $value;
@@ -51,20 +59,36 @@ sub read_protocol {
             $value_to_comment_expanded{$value} ||= $comment;
             push @fill, $name if substr($name, -4) eq '_LOW';
 
-            if ( $value < 128 ) {
+            if ( $value < 128 && !($name=~/_LOW/ or $name=~/_HIGH/)) {
                 $meta[$value]{name}= $name;
                 $meta[$value]{value}= $value;
                 $meta[$value]{type_name}= $name;
                 $meta[$value]{type_value}= $value;
                 $meta[$value]{comment}= $comment if defined $comment;
             }
+            $define{$full_name}= $value;
+        } elsif (s!^#define (SRL_\w+)\s+!!) {
+            my $def= $1;
+            s!/\*.*?(?:\*/|$)!!m;
+            s!\(U8\)!!g;
+            s!(SRL_\w+)!
+                $define{$1} // die "Unknown define '$1'";
+            !ge;
+            s!\A\s+!!;
+            s!\s+\z!!;
+            my $val;
+            my $code= "\$val= $_; 1";
+            eval $code or die "Failed to eval $code (from $orig): $@";
+            $define{$def}= $val;
+            $define_is_str{$def}= 1 if /[""]/;
         }
     }
     close $fh;
-    fill_ranges($_) for @fill;
+    fill_range($_) for @fill;
     foreach my $pfx (keys %name_to_value_expanded) {
         $max_name_length= length($pfx) if $max_name_length < length($pfx);
     }
+    #print Data::Dumper->new([\%define, \%define_is_str])->Useqq(1)->Sortkeys(1)->Dump();
 }
 
 sub open_swap {
@@ -81,41 +105,75 @@ sub open_swap {
 sub replace_block {
     my ($file,$blob)= @_;
     my ($in,$out)= open_swap($file);
-    while (<$in>) {
-        print $out $_;
-        last if /^=for autoupdater start/ || /^# start autoupdated section/;
-    }
-    $blob=~s/\s+$//mg;
-    print $out "\n$blob\n\n";
-    while (<$in>) {
-        if (/^=for autoupdater stop/ || /^# stop autoupdated section/) {
+    my $gotit;
+    READ: {
+
+        while (<$in>) {
+            print $out $_;
+            last if $gotit= (/^=for autoupdater start/ || /^# start autoupdated section/);
+        }
+
+        unless ($gotit) {
+            warn "didnt find autoupdater start!\n";
+            last READ;
+        }
+
+        $blob =~ s/[ \t]+$//mg;
+        $blob =~ s/\s+\z//;
+
+        print $out "\n$blob\n\n";
+
+        while (<$in>) {
+            if (/^=for autoupdater stop/ || /^# stop autoupdated section/) {
+                print $out $_;
+                $gotit= 0;
+                last;
+            }
+        }
+
+        if ($gotit) {
+            warn "didnt find autoupdater start!\n";
+            last READ;
+        }
+
+        while (<$in>) {
             print $out $_;
-            last;
         }
-    }
-    while (<$in>) {
-        print $out $_;
     }
     close $out;
     close $in;
+    return;
 }
-sub update_buildtools {
-    my $dump= Data::Dumper->new([\@meta],['*TAG_INFO_ARRAY'])->Indent(1)->Dump();
+
+sub update_constants {
+    my $dump= Data::Dumper->new([\@meta],['*TAG_INFO_ARRAY'])->Sortkeys(1)->Useqq(1)->Indent(1)->Dump();
     $dump =~ s/^(\s*)\{/$1# autoupdated by $0 do not modify directly!\n$1\{/mg;
-    return replace_block(
-        "Perl/shared/inc/Sereal/BuildTools.pm",
-        join "\n",
-            "our (%TAG_INFO_HASH, \@TAG_INFO_ARRAY);",
-            $dump,
-            "\$TAG_INFO_HASH{chr \$_}= \$TAG_INFO_ARRAY[\$_] for 0 .. 127;",
-            "push \@EXPORT_OK, qw(%TAG_INFO_HASH \@TAG_INFO_ARRAY);",
-    )
+    my $defines= Data::Dumper->new([\%define],['*DEFINE'])->Sortkeys(1)->Useqq(1)->Indent(1)->Dump;
+    $defines=~s/^/    /mg;
+
+    foreach my $mod_suffix (qw(Encoder Decoder)) {
+        replace_block(
+            "Perl/$mod_suffix/lib/Sereal/$mod_suffix/Constants.pm",
+            join "\n",
+                "BEGIN {",
+                $defines,
+                "}",
+                "",
+                "use constant \\%DEFINE;",
+                "push \@EXPORT_OK, keys %DEFINE;",
+                $dump,
+                "\$TAG_INFO_HASH{chr \$_}= \$TAG_INFO_ARRAY[\$_] for 0 .. 127;",
+                "push \@EXPORT_OK, qw(%TAG_INFO_HASH \@TAG_INFO_ARRAY);",
+        )
+    }
 }
-sub update_srl_decoder_h {
-    replace_block("Perl/Decoder/srl_decoder.h",
+
+sub update_srl_taginfo_h {
+    replace_block("Perl/shared/srl_taginfo.h",
         join("\n",
             "* NOTE this section is autoupdated by $0",
             "*/",
+            "",
             "static const char * const tag_name[] = {",
             ( map {
                 my $str= Data::Dumper::qquote(chr($_));
@@ -124,16 +182,37 @@ sub update_srl_decoder_h {
                     $max_name_length+3, qq("$value_to_name_expanded{$_}") . ($_==127 ? " " : ","), $str, $_, $_, $_
             } 0 .. 127 ),
             "};",
+            "",
+            (
+                map {
+                    sprintf "#define SRL_HDR_%-*s %3d",
+                        $max_name_length+3, $_->{name}, $_->{value}
+                } grep { $_->{masked} } @meta
+            ),
+            "",
+            ( map {
+                my $n = $_;
+                my $v = $range{$n};
+                my $c =
+                        join "    \\\n   ",
+                        "#define CASE_SRL_HDR_$n",
+                        join ":    \\\n   ",
+                        map { "case SRL_HDR_$_->{name}" } @$v;
+
+                $c."\n\n";
+            } sort keys %range ),
+            "",
             "/*",
             "* NOTE the above section is auto-updated by $0",
         )
-    )
+    );
 }
 
+
 sub update_JavaSerealHeader {
     my $declarations = "* NOTE this section is autoupdated by $0 */\n";
 
-    for my $name (sort { $name_to_value{$a} <=> $name_to_value{$b} } keys %name_to_value) {
+    for my $name (sort { $name_to_value{$a} <=> $name_to_value{$b} || $a cmp $b } keys %name_to_value) {
         my $byte = $name_to_value{$name};
         my $decl = sprintf("static final byte SRL_HDR_%-*s = (byte) %3d;", $max_name_length, $name, $byte);
         $declarations .= sprintf("\t%s /* %3d 0x%02x 0b%08b %s */\n",
@@ -172,8 +251,8 @@ chomp($git_dir);
 chdir "$git_dir/.."
     or die "Failed to chdir to root of repo '$git_dir/..': $!";
 read_protocol();
-update_buildtools();
-update_srl_decoder_h();
+update_constants();
+update_srl_taginfo_h();
 update_table("sereal_spec.pod");
 update_table("Perl/shared/srl_protocol.h");
 update_JavaSerealHeader();
